@@ -233,11 +233,7 @@ pub fn derive_sawp_ffi(input: proc_macro::TokenStream) -> proc_macro::TokenStrea
 
 fn impl_sawp_ffi(ast: &syn::DeriveInput) -> TokenStream {
     let name = &ast.ident;
-    let ffi_metas: Vec<syn::NestedMeta> = ast
-        .attrs
-        .iter()
-        .flat_map(|attr| get_ffi_meta(attr))
-        .collect();
+    let ffi_metas: Vec<syn::NestedMeta> = ast.attrs.iter().flat_map(get_ffi_meta).collect();
     let prefix = get_ffi_prefix(&ffi_metas);
     match &ast.data {
         syn::Data::Struct(data) => gen_struct_accessors(&prefix, name, data),
@@ -281,13 +277,28 @@ fn is_string_type(ty: &syn::Type) -> bool {
 /// ```
 fn split_generic(ty: &syn::Type) -> Option<(syn::Ident, syn::Ident)> {
     if let syn::Type::Path(ty) = ty {
-        let segment = &ty.path.segments.first().expect("Path with no segments");
-        if let syn::PathArguments::AngleBracketed(arguments) = &segment.arguments {
-            if arguments.args.len() == 1 {
-                if let syn::GenericArgument::Type(syn::Type::Path(arg)) = &arguments.args[0] {
-                    if let Some(inner_ident) = arg.path.get_ident() {
-                        return Some((segment.ident.clone(), (*inner_ident).clone()));
-                    }
+        if let Some((ident, arg)) = split_generic_helper(ty) {
+            if let Some(inner_ident) = arg.path.get_ident() {
+                return Some((ident, (*inner_ident).clone()));
+            }
+        }
+    }
+    None
+}
+
+/// Get outer, mid and inner types of a generic data type
+/// Returns `None` if there isn't exactly 2 generics
+///
+/// # Example
+/// ```ignore
+/// Vec<Vec<u8>> -> Some(Vec, Vec, u8)
+/// ```
+fn split_generic_multi(ty: &syn::Type) -> Option<(syn::Ident, syn::Ident, syn::Ident)> {
+    if let syn::Type::Path(ty) = ty {
+        if let Some((ident_1, arg)) = split_generic_helper(ty) {
+            if let Some((ident_2, arg)) = split_generic_helper(&arg) {
+                if let Some(ident_3) = arg.path.get_ident() {
+                    return Some((ident_1, ident_2, (*ident_3).clone()));
                 }
             }
         }
@@ -295,6 +306,18 @@ fn split_generic(ty: &syn::Type) -> Option<(syn::Ident, syn::Ident)> {
     None
 }
 
+fn split_generic_helper(ty: &syn::TypePath) -> Option<(syn::Ident, syn::TypePath)> {
+    let segment = &ty.path.segments.first().expect("Path with no segments");
+    if let syn::PathArguments::AngleBracketed(arguments) = &segment.arguments {
+        if arguments.args.len() == 1 {
+            if let syn::GenericArgument::Type(syn::Type::Path(arg)) = &arguments.args[0] {
+                return Some((segment.ident.clone(), arg.clone()));
+            }
+        }
+    }
+    None
+}
+
 /// Assign ptr to *ptr after checking it is not null
 fn deref(field_name: &syn::Ident) -> TokenStream {
     quote! {
@@ -386,11 +409,8 @@ fn gen_struct_accessors(
                 _ => continue,
             }
 
-            let ffi_metas: Vec<syn::NestedMeta> = field
-                .attrs
-                .iter()
-                .flat_map(|attr| get_ffi_meta(attr))
-                .collect();
+            let ffi_metas: Vec<syn::NestedMeta> =
+                field.attrs.iter().flat_map(get_ffi_meta).collect();
             if has_ffi_skip_meta(&ffi_metas) {
                 continue;
             }
@@ -501,6 +521,23 @@ fn gen_field_accessor(
                 });
             }
         }
+    } else if let Some((outer, mid, inner)) = split_generic_multi(ty) {
+        if outer.to_string().as_str() == "Vec"
+            && mid.to_string().as_str() == "Vec"
+            && inner.to_string().as_str() == "u8"
+        {
+            let idx_name = format_ident!("{}_ptr_to_idx", func_name);
+            accessors.extend(quote! {
+                /// Get ptr to member of `#struct_variable.#field` at index
+                /// # Safety
+                /// function will panic if called with null or an index outside bounds
+                #[no_mangle]
+                pub unsafe extern "C" fn #idx_name(#struct_variable: *const #struct_name, n: usize) -> *const #inner {
+                    #deref_variable
+                    (#ret_var[n]).as_ptr()
+                }
+            });
+        }
     }
     accessors
 }
@@ -627,11 +664,8 @@ fn gen_enum_accessors(
             match &variant.fields {
                 syn::Fields::Named(fields) => {
                     for field in &fields.named {
-                        let ffi_metas: Vec<syn::NestedMeta> = field
-                            .attrs
-                            .iter()
-                            .flat_map(|attr| get_ffi_meta(attr))
-                            .collect();
+                        let ffi_metas: Vec<syn::NestedMeta> =
+                            field.attrs.iter().flat_map(get_ffi_meta).collect();
                         if has_ffi_skip_meta(&ffi_metas) {
                             continue;
                         }
@@ -648,11 +682,8 @@ fn gen_enum_accessors(
                 }
                 syn::Fields::Unnamed(fields) => {
                     for (i, field) in (&fields.unnamed).into_iter().enumerate() {
-                        let ffi_metas: Vec<syn::NestedMeta> = field
-                            .attrs
-                            .iter()
-                            .flat_map(|attr| get_ffi_meta(attr))
-                            .collect();
+                        let ffi_metas: Vec<syn::NestedMeta> =
+                            field.attrs.iter().flat_map(get_ffi_meta).collect();
                         if has_ffi_skip_meta(&ffi_metas) {
                             continue;
                         }
@@ -997,6 +1028,28 @@ mod tests {
         impl_sawp_ffi(&parsed);
     }
 
+    #[test]
+    fn test_macro_struct_multi() {
+        let input = r#"
+            #[sawp_ffi(prefix = "sawp")]
+            pub struct MyStructMulti {
+                pub num: usize,
+                #[sawp_ffi(copy)]
+                pub version: Version,
+                #[sawp_ffi(flag = "u8")]
+                pub file_type: Flags<FileType>,
+                private: usize,
+                #[sawp_ffi(skip)]
+                skipped: usize,
+                pub complex: Vec<Vec<u8>>,
+                pub string: String,
+                pub option: Option<u8>,
+            }
+	"#;
+        let parsed: syn::DeriveInput = syn::parse_str(input).unwrap();
+        impl_sawp_ffi(&parsed);
+    }
+
     #[test]
     #[should_panic(expected = "expects string literal")]
     fn test_macro_prefix_panic() {
